"use strict";
/*
 * Copyright 2020 The Kale Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const RPCUtils_1 = require("./RPCUtils");
const Utils_1 = require("./Utils");
const LeftPanelWidget_1 = require("../widgets/LeftPanelWidget");
const NotebookUtils_1 = require("./NotebookUtils");
const VolumesPanel_1 = require("../widgets/VolumesPanel");
const CellUtils_1 = require("./CellUtils");
var RUN_CELL_STATUS;
(function (RUN_CELL_STATUS) {
    RUN_CELL_STATUS["OK"] = "ok";
    RUN_CELL_STATUS["ERROR"] = "error";
})(RUN_CELL_STATUS || (RUN_CELL_STATUS = {}));
class Commands {
    constructor(notebook, kernel) {
        this.snapshotNotebook = () => __awaiter(this, void 0, void 0, function* () {
            return yield RPCUtils_1._legacy_executeRpcAndShowRPCError(this._notebook, this._kernel, 'rok.snapshot_notebook');
        });
        this.getSnapshotProgress = (task_id, ms) => __awaiter(this, void 0, void 0, function* () {
            const task = yield RPCUtils_1._legacy_executeRpcAndShowRPCError(this._notebook, this._kernel, 'rok.get_task', {
                task_id,
            });
            if (ms) {
                yield Utils_1.wait(ms);
            }
            return task;
        });
        this.runSnapshotProcedure = (onUpdate) => __awaiter(this, void 0, void 0, function* () {
            const showSnapshotProgress = true;
            const snapshot = yield this.snapshotNotebook();
            const taskId = snapshot.task.id;
            let task = yield this.getSnapshotProgress(taskId);
            onUpdate({ task, showSnapshotProgress });
            while (!['success', 'error', 'canceled'].includes(task.status)) {
                task = yield this.getSnapshotProgress(taskId, 1000);
                onUpdate({ task });
            }
            if (task.status === 'success') {
                console.log('Snapshotting successful!');
                return task;
            }
            else if (task.status === 'error') {
                console.error('Snapshotting failed');
                console.error('Stopping the deployment...');
            }
            else if (task.status === 'canceled') {
                console.error('Snapshotting canceled');
                console.error('Stopping the deployment...');
            }
            return null;
        });
        this.replaceClonedVolumes = (bucket, obj, version, volumes) => __awaiter(this, void 0, void 0, function* () {
            return yield RPCUtils_1._legacy_executeRpcAndShowRPCError(this._notebook, this._kernel, 'rok.replace_cloned_volumes', {
                bucket,
                obj,
                version,
                volumes,
            });
        });
        this.getMountedVolumes = (currentNotebookVolumes) => __awaiter(this, void 0, void 0, function* () {
            let notebookVolumes = yield RPCUtils_1._legacy_executeRpcAndShowRPCError(this._notebook, this._kernel, 'nb.list_volumes');
            let availableVolumeTypes = VolumesPanel_1.SELECT_VOLUME_TYPES.map(t => {
                return t.value === 'snap' ? Object.assign({}, t, { invalid: false }) : t;
            });
            if (notebookVolumes) {
                notebookVolumes = notebookVolumes.map(volume => {
                    const sizeGroup = VolumesPanel_1.SELECT_VOLUME_SIZE_TYPES.filter(s => volume.size >= s.base)[0];
                    volume.size = Math.ceil(volume.size / sizeGroup.base);
                    volume.size_type = sizeGroup.value;
                    volume.annotations = [];
                    return volume;
                });
                availableVolumeTypes = availableVolumeTypes.map(t => {
                    return t.value === 'clone' ? Object.assign({}, t, { invalid: false }) : t;
                });
            }
            else {
                notebookVolumes = currentNotebookVolumes;
            }
            return {
                notebookVolumes,
                selectVolumeTypes: availableVolumeTypes,
            };
        });
        this.unmarshalData = (nbFileName) => __awaiter(this, void 0, void 0, function* () {
            const cmd = `from kale.rpc.nb import unmarshal_data as __kale_rpc_unmarshal_data\n` +
                `locals().update(__kale_rpc_unmarshal_data("${nbFileName}"))`;
            console.log('Executing command: ' + cmd);
            yield NotebookUtils_1.default.sendKernelRequestFromNotebook(this._notebook, cmd, {});
        });
        this.getBaseImage = () => __awaiter(this, void 0, void 0, function* () {
            let baseImage = null;
            try {
                baseImage = yield RPCUtils_1._legacy_executeRpc(this._notebook, this._kernel, 'nb.get_base_image');
            }
            catch (error) {
                if (error instanceof RPCUtils_1.RPCError) {
                    console.warn('Kale is not running in a Notebook Server', error.error);
                }
                else {
                    throw error;
                }
            }
            return baseImage;
        });
        this.getExperiments = (experiment, experimentName) => __awaiter(this, void 0, void 0, function* () {
            let experimentsList = yield RPCUtils_1._legacy_executeRpcAndShowRPCError(this._notebook, this._kernel, 'kfp.list_experiments');
            if (experimentsList) {
                experimentsList.push(LeftPanelWidget_1.NEW_EXPERIMENT);
            }
            else {
                experimentsList = [LeftPanelWidget_1.NEW_EXPERIMENT];
            }
            // Fix experiment metadata
            let newExperiment = null;
            let selectedExperiments = experimentsList.filter(e => e.id === experiment.id ||
                e.name === experiment.name ||
                e.name === experimentName);
            if (selectedExperiments.length === 0 ||
                selectedExperiments[0].id === LeftPanelWidget_1.NEW_EXPERIMENT.id) {
                let name = experimentsList[0].name;
                if (name === LeftPanelWidget_1.NEW_EXPERIMENT.name) {
                    name = experiment.name !== '' ? experiment.name : experimentName;
                }
                newExperiment = Object.assign({}, experimentsList[0], { name: name });
            }
            else {
                newExperiment = selectedExperiments[0];
            }
            return {
                experiments: experimentsList,
                experiment: newExperiment,
                experiment_name: newExperiment.name,
            };
        });
        this.validateMetadata = (notebookPath, metadata, onUpdate) => __awaiter(this, void 0, void 0, function* () {
            onUpdate({ showValidationProgress: true });
            const validateNotebookArgs = {
                source_notebook_path: notebookPath,
                notebook_metadata_overrides: metadata,
            };
            const validateNotebook = yield RPCUtils_1._legacy_executeRpcAndShowRPCError(this._notebook, this._kernel, 'nb.validate_notebook', validateNotebookArgs);
            if (!validateNotebook) {
                onUpdate({ notebookValidation: false });
                return false;
            }
            onUpdate({ notebookValidation: true });
            return true;
        });
        /**
         * Analyse the current metadata and produce some warning to be shown
         * under the compilation task
         * @param metadata Notebook metadata
         */
        this.getCompileWarnings = (metadata) => {
            let warningContent = [];
            // in case the notebook's docker base image is different than the default
            // one (e.g. the one detected in the Notebook Server), alert the user
            if (LeftPanelWidget_1.DefaultState.metadata.docker_image !== '' &&
                metadata.docker_image !== LeftPanelWidget_1.DefaultState.metadata.docker_image) {
                warningContent.push('The image you used to create the notebook server is different ' +
                    'from the image you have selected for your pipeline.', '', 'Your Kubeflow pipeline will use the following image: <pre><b>' +
                    metadata.docker_image +
                    '</b></pre>', 'You created the notebook server using the following image: <pre><b>' +
                    LeftPanelWidget_1.DefaultState.metadata.docker_image +
                    '</b></pre>', '', "To use this notebook server's image as base image" +
                    ' for the pipeline steps, delete the existing docker image' +
                    ' from the Advanced Settings section.');
            }
            return warningContent;
        };
        // todo: docManager needs to be passed to deploysProgress during init
        // todo: autosnapshot will become part of metadata
        // todo: deployDebugMessage will be removed (the "Debug" toggle is of no use
        //  anymore
        this.compilePipeline = (notebookPath, metadata, docManager, deployDebugMessage, autosnapshot, onUpdate) => __awaiter(this, void 0, void 0, function* () {
            // after parsing and validating the metadata, show warnings (if necessary)
            const compileWarnings = this.getCompileWarnings(metadata);
            onUpdate({ showCompileProgress: true, docManager: docManager });
            if (compileWarnings.length) {
                onUpdate({ compileWarnings });
            }
            const compileNotebookArgs = {
                source_notebook_path: notebookPath,
                notebook_metadata_overrides: metadata,
                debug: deployDebugMessage,
                auto_snapshot: autosnapshot,
            };
            const compileNotebook = yield RPCUtils_1._legacy_executeRpcAndShowRPCError(this._notebook, this._kernel, 'nb.compile_notebook', compileNotebookArgs);
            if (!compileNotebook) {
                onUpdate({ compiledPath: 'error' });
                yield NotebookUtils_1.default.showMessage('Operation Failed', [
                    'Could not compile pipeline.',
                ]);
            }
            else {
                // Pass to the deploy progress the path to the generated py script:
                // compileNotebook is the name of the tar package, that generated in the
                // workdir. Instead, the python script has a slightly different name and
                // is generated in the same directory where the notebook lives.
                onUpdate({
                    compiledPath: compileNotebook.pipeline_package_path.replace('pipeline.yaml', 'kale.py'),
                });
            }
            return compileNotebook;
        });
        this.uploadPipeline = (compiledPackagePath, compiledPipelineMetadata, onUpdate) => __awaiter(this, void 0, void 0, function* () {
            onUpdate({ showUploadProgress: true });
            const uploadPipelineArgs = {
                pipeline_package_path: compiledPackagePath,
                pipeline_metadata: compiledPipelineMetadata,
                overwrite: false,
            };
            let uploadPipeline = yield RPCUtils_1._legacy_executeRpcAndShowRPCError(this._notebook, this._kernel, 'kfp.upload_pipeline', uploadPipelineArgs);
            let result = true;
            if (!uploadPipeline) {
                onUpdate({ showUploadProgress: false, pipeline: false });
                return uploadPipeline;
            }
            if (uploadPipeline && uploadPipeline.already_exists) {
                // show dialog to ask user if they want to overwrite the existing pipeline
                result = yield NotebookUtils_1.default.showYesNoDialog('Pipeline Upload Failed', [
                    'Pipeline with name ' +
                        compiledPipelineMetadata.pipeline_name +
                        ' already exists. ',
                    'Would you like to overwrite it?',
                ]);
                // OVERWRITE EXISTING PIPELINE
                if (result) {
                    uploadPipelineArgs.overwrite = true;
                    uploadPipeline = yield RPCUtils_1._legacy_executeRpcAndShowRPCError(this._notebook, this._kernel, 'kfp.upload_pipeline', uploadPipelineArgs);
                }
                else {
                    onUpdate({ pipeline: false });
                }
            }
            if (uploadPipeline && result) {
                onUpdate({ pipeline: uploadPipeline });
            }
            return uploadPipeline;
        });
        this.runKatib = (notebookPath, metadata, pipelineId, onUpdate) => __awaiter(this, void 0, void 0, function* () {
            onUpdate({ showKatibKFPExperiment: true });
            // create a new experiment, using the base name of the currently
            // selected one
            const newExpName = metadata.experiment.name +
                '-' +
                Math.random()
                    .toString(36)
                    .slice(2, 7);
            // create new KFP experiment
            let kfpExperiment;
            try {
                kfpExperiment = yield RPCUtils_1._legacy_executeRpc(this._notebook, this._kernel, 'kfp.create_experiment', {
                    experiment_name: newExpName,
                });
                onUpdate({ katibKFPExperiment: kfpExperiment });
            }
            catch (error) {
                onUpdate({
                    showKatibProgress: false,
                    katibKFPExperiment: { id: 'error', name: 'error' },
                });
                throw error;
            }
            onUpdate({ showKatibProgress: true });
            const runKatibArgs = {
                pipeline_id: pipelineId,
                pipeline_metadata: Object.assign({}, metadata, { experiment_name: kfpExperiment.name }),
                output_path: notebookPath.substring(0, notebookPath.lastIndexOf('/')),
            };
            let katibExperiment = null;
            try {
                katibExperiment = yield RPCUtils_1._legacy_executeRpc(this._notebook, this._kernel, 'katib.create_katib_experiment', runKatibArgs);
            }
            catch (error) {
                onUpdate({ katib: { status: 'error' } });
                throw error;
            }
            return katibExperiment;
        });
        this.runPipeline = (pipelineId, compiledPipelineMetadata, onUpdate) => __awaiter(this, void 0, void 0, function* () {
            onUpdate({ showRunProgress: true });
            const runPipelineArgs = {
                pipeline_metadata: compiledPipelineMetadata,
                pipeline_id: pipelineId,
            };
            const runPipeline = yield RPCUtils_1._legacy_executeRpcAndShowRPCError(this._notebook, this._kernel, 'kfp.run_pipeline', runPipelineArgs);
            if (runPipeline) {
                onUpdate({ runPipeline });
            }
            else {
                onUpdate({ showRunProgress: false, runPipeline: false });
            }
            return runPipeline;
        });
        this.resumeStateIfExploreNotebook = (notebookPath) => __awaiter(this, void 0, void 0, function* () {
            const exploration = yield RPCUtils_1._legacy_executeRpcAndShowRPCError(this._notebook, this._kernel, 'nb.explore_notebook', { source_notebook_path: notebookPath });
            if (!exploration || !exploration.is_exploration) {
                return;
            }
            NotebookUtils_1.default.clearCellOutputs(this._notebook);
            let runCellResponse = yield NotebookUtils_1.default.runGlobalCells(this._notebook);
            if (runCellResponse.status === RUN_CELL_STATUS.OK) {
                // unmarshalData runs in the same kernel as the .ipynb, so it requires the
                // filename
                yield this.unmarshalData(notebookPath.split('/').pop());
                const cell = CellUtils_1.default.getCellByStepName(this._notebook, exploration.step_name);
                const title = 'Notebook Exploration';
                const message = [`Resuming notebook at step: "${exploration.step_name}"`];
                if (cell) {
                    NotebookUtils_1.default.selectAndScrollToCell(this._notebook, cell);
                }
                else {
                    message.push(`ERROR: Could not retrieve step's position.`);
                }
                yield NotebookUtils_1.default.showMessage(title, message);
            }
            else {
                yield NotebookUtils_1.default.showMessage('Notebook Exploration', [
                    `Executing "${runCellResponse.cellType}" cell failed.\n` +
                        `Resuming notebook at cell index ${runCellResponse.cellIndex}.`,
                    `Error name: ${runCellResponse.ename}`,
                    `Error value: ${runCellResponse.evalue}`,
                ]);
            }
            yield RPCUtils_1._legacy_executeRpcAndShowRPCError(this._notebook, this._kernel, 'nb.remove_marshal_dir', {
                source_notebook_path: notebookPath,
            });
        });
        this._notebook = notebook;
        this._kernel = kernel;
    }
    pollRun(runPipeline, onUpdate) {
        RPCUtils_1._legacy_executeRpcAndShowRPCError(this._notebook, this._kernel, 'kfp.get_run', {
            run_id: runPipeline.id,
        }).then(run => {
            onUpdate({ runPipeline: run });
            if (run && (run.status === 'Running' || run.status === null)) {
                setTimeout(() => this.pollRun(run, onUpdate), 2000);
            }
        });
    }
    pollKatib(katibExperiment, onUpdate) {
        const getExperimentArgs = {
            experiment: katibExperiment.name,
            namespace: katibExperiment.namespace,
        };
        RPCUtils_1._legacy_executeRpcAndShowRPCError(this._notebook, this._kernel, 'katib.get_experiment', getExperimentArgs).then(katib => {
            if (!katib) {
                // could not get the experiment
                onUpdate({ katib: { status: 'error' } });
                return;
            }
            onUpdate({ katib });
            if (katib && katib.status !== 'Succeeded' && katib.status !== 'Failed') {
                setTimeout(() => this.pollKatib(katibExperiment, onUpdate), 5000);
            }
        });
    }
}
exports.default = Commands;
