"use strict";
/*
 * Copyright 2019-2020 The Kale Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const yaml = require("js-yaml");
const core_1 = require("@material-ui/core");
const Close_1 = require("@material-ui/icons/Close");
const Launch_1 = require("@material-ui/icons/Launch");
const Error_1 = require("@material-ui/icons/Error");
const Help_1 = require("@material-ui/icons/Help");
const Schedule_1 = require("@material-ui/icons/Schedule");
const SkipNext_1 = require("@material-ui/icons/SkipNext");
const CheckCircle_1 = require("@material-ui/icons/CheckCircle");
const Cancel_1 = require("@material-ui/icons/Cancel");
const Warning_1 = require("@material-ui/icons/Warning");
const Info_1 = require("@material-ui/icons/Info");
const statusRunning_1 = require("../../icons/statusRunning");
const statusTerminated_1 = require("../../icons/statusTerminated");
const NotebookUtils_1 = require("../../lib/NotebookUtils");
var KatibExperimentStatus;
(function (KatibExperimentStatus) {
    KatibExperimentStatus["CREATED"] = "Created";
    KatibExperimentStatus["RUNNING"] = "Running";
    KatibExperimentStatus["RESTARTING"] = "Restarting";
    KatibExperimentStatus["SUCCEEDED"] = "Succeeded";
    KatibExperimentStatus["FAILED"] = "Failed";
    KatibExperimentStatus["UNKNOWN"] = "Unknown";
})(KatibExperimentStatus || (KatibExperimentStatus = {}));
// From kubeflow/pipelines repo
var PipelineStatus;
(function (PipelineStatus) {
    PipelineStatus["ERROR"] = "Error";
    PipelineStatus["FAILED"] = "Failed";
    PipelineStatus["PENDING"] = "Pending";
    PipelineStatus["RUNNING"] = "Running";
    PipelineStatus["SKIPPED"] = "Skipped";
    PipelineStatus["SUCCEEDED"] = "Succeeded";
    PipelineStatus["TERMINATING"] = "Terminating";
    PipelineStatus["TERMINATED"] = "Terminated";
    PipelineStatus["UNKNOWN"] = "Unknown";
})(PipelineStatus || (PipelineStatus = {}));
const color = {
    // From kubeflow/pipelines repo
    activeBg: '#eaf1fd',
    alert: '#f9ab00',
    background: '#fff',
    blue: '#4285f4',
    disabledBg: '#ddd',
    divider: '#e0e0e0',
    errorBg: '#fbe9e7',
    errorText: '#d50000',
    foreground: '#000',
    graphBg: '#f2f2f2',
    grey: '#5f6368',
    inactive: '#5f6368',
    lightGrey: '#eee',
    lowContrast: '#80868b',
    secondaryText: 'rgba(0, 0, 0, .88)',
    separator: '#e8e8e8',
    strong: '#202124',
    success: '#34a853',
    successWeak: '#e6f4ea',
    terminated: '#80868b',
    theme: '#1a73e8',
    themeDarker: '#0b59dc',
    warningBg: '#f9f9e1',
    warningText: '#ee8100',
    weak: '#9aa0a6',
    // From Rok repo
    canceled: '#ff992a',
};
exports.DeployProgress = props => {
    const getWarningBadge = (title, content) => {
        return (content && (React.createElement("a", { onClick: _ => {
                NotebookUtils_1.default.showMessage(title, content);
            } },
            React.createElement(Warning_1.default, { style: { color: color.alert, height: 18, width: 18 } }))));
    };
    const getInfoBadge = (title, content) => {
        return (content && (React.createElement("a", { onClick: _ => {
                NotebookUtils_1.default.showMessage(title, content);
            } },
            React.createElement(Info_1.default, { style: { color: color.blue, height: 18, width: 18 } }))));
    };
    const getKatibBestResultInfo = (katib) => {
        let optimal = katib ? katib.currentOptimalTrial : null;
        // currentOptimalTrial is _never_ null,
        // so if there's no best trial so far we don't show the object
        return optimal && optimal.bestTrialName
            ? [yaml.safeDump(optimal)]
            : [
                'There are no results yet',
                'To have a result, there must be at least one successful trial',
            ];
    };
    const getSnapshotLink = (task) => {
        if (!task.result || !task.result.event) {
            return '#';
        }
        return `${window.location.origin}/_/rok/buckets/${task.bucket}/files/${task.result.event.object}/versions/${task.result.event.version}`;
    };
    const getTaskLink = (task) => {
        return `${window.location.origin}/_/rok/buckets/${task.bucket}/tasks/${task.id}`;
    };
    const getUploadLink = (pipeline) => {
        // link: /_/pipeline/#/pipelines/details/<id>
        // id = uploadPipeline.pipeline.id
        if (!pipeline.pipeline || !pipeline.pipeline.id) {
            return '#';
        }
        return `${window.location.origin}/_/pipeline/#/pipelines/details/${pipeline.pipeline.id}`;
    };
    const getRunLink = (pipeline) => {
        // link: /_/pipeline/#/runs/details/<id>
        // id = runPipeline.id
        if (!pipeline.id) {
            return '#';
        }
        return `${window.location.origin}/_/pipeline/#/runs/details/${pipeline.id}`;
    };
    const getRunText = (pipeline) => {
        switch (pipeline.status) {
            case null:
            case 'Running':
                return 'View';
            case 'Terminating':
            case 'Failed':
                return pipeline.status;
            default:
                return 'Done';
        }
    };
    const getRunComponent = (pipeline) => {
        let title = 'Unknown status';
        let IconComponent = Help_1.default;
        let iconColor = '#5f6368';
        switch (pipeline.status) {
            case PipelineStatus.ERROR:
                IconComponent = Error_1.default;
                iconColor = color.errorText;
                // title = 'Error';
                break;
            case PipelineStatus.FAILED:
                IconComponent = Error_1.default;
                iconColor = color.errorText;
                // title = 'Failed';
                break;
            case PipelineStatus.PENDING:
                IconComponent = Schedule_1.default;
                iconColor = color.weak;
                // title = 'Pendig';
                break;
            case PipelineStatus.RUNNING:
                IconComponent = statusRunning_1.default;
                iconColor = color.blue;
                // title = 'Running';
                break;
            case PipelineStatus.TERMINATING:
                IconComponent = statusRunning_1.default;
                iconColor = color.blue;
                // title = 'Terminating';
                break;
            case PipelineStatus.SKIPPED:
                IconComponent = SkipNext_1.default;
                // title = 'Skipped';
                break;
            case PipelineStatus.SUCCEEDED:
                IconComponent = CheckCircle_1.default;
                iconColor = color.success;
                // title = 'Succeeded';
                break;
            case PipelineStatus.TERMINATED:
                IconComponent = statusTerminated_1.default;
                iconColor = color.terminated;
                // title = 'Terminated';
                break;
            case PipelineStatus.UNKNOWN:
                break;
            default:
                console.error('pipeline status:', pipeline.status);
        }
        return (React.createElement(React.Fragment, null,
            getRunText(pipeline),
            React.createElement(IconComponent, { style: { color: iconColor, height: 18, width: 18 } })));
    };
    const getKatibLink = (katib) => {
        // link: /_/katib/#/katib/hp_monitor/<namespace>/<name>
        if (!katib.name && !katib.namespace) {
            return '#';
        }
        return `${window.location.origin}/_/katib/#/katib/hp_monitor/${katib.namespace}/${katib.name}`;
    };
    const getKatibText = (katib) => {
        switch (katib.status) {
            case KatibExperimentStatus.FAILED:
                return 'Failed';
            case KatibExperimentStatus.SUCCEEDED:
                return 'Done';
            default:
                return 'View';
        }
    };
    const getKatibComponent = (katib) => {
        let IconComponent = Help_1.default;
        let iconColor = '#5f6368';
        switch (katib.status) {
            case KatibExperimentStatus.FAILED:
                IconComponent = Error_1.default;
                iconColor = color.errorText;
                break;
            case KatibExperimentStatus.CREATED:
            case KatibExperimentStatus.RUNNING:
            case KatibExperimentStatus.RESTARTING:
                IconComponent = statusRunning_1.default;
                iconColor = color.blue;
                break;
            case KatibExperimentStatus.SUCCEEDED:
                IconComponent = CheckCircle_1.default;
                iconColor = color.success;
                break;
            default:
                break;
        }
        return (React.createElement(React.Fragment, null,
            getKatibText(katib),
            React.createElement(IconComponent, { style: { color: iconColor, height: 18, width: 18 } })));
    };
    const getKatibKfpExperimentLink = (experimentId) => {
        // link: /_/pipeline/#/experiments/details/<ud>
        if (!experimentId) {
            return '#';
        }
        return `${window.location.origin}/_/pipeline/#/experiments/details/${experimentId}`;
    };
    const getSnapshotTpl = () => {
        if (!props.task) {
            return (React.createElement(React.Fragment, null,
                "Unknown status",
                React.createElement(Help_1.default, { style: { color: color.terminated, height: 18, width: 18 } })));
        }
        if (!['success', 'error', 'canceled'].includes(props.task.status)) {
            const progress = props.task.progress || 0;
            return (React.createElement(core_1.LinearProgress, { variant: "determinate", color: "primary", value: progress }));
        }
        let getLink = () => '#';
        let message = props.task.message;
        let IconComponent = Help_1.default;
        let iconColor = color.blue;
        switch (props.task.status) {
            case 'success':
                getLink = getSnapshotLink;
                message = 'Done';
                IconComponent = Launch_1.default;
                break;
            case 'error':
                getLink = getTaskLink;
                IconComponent = Error_1.default;
                iconColor = color.errorText;
                break;
            case 'canceled':
                IconComponent = Cancel_1.default;
                getLink = getTaskLink;
                iconColor = color.canceled;
                break;
        }
        return (React.createElement(React.Fragment, null,
            React.createElement("a", { href: getLink(props.task), target: "_blank", rel: "noopener noreferrer" },
                message,
                React.createElement(IconComponent, { style: { color: iconColor, height: 18, width: 18 } }))));
    };
    let validationTpl;
    if (props.notebookValidation === true) {
        validationTpl = (React.createElement(React.Fragment, null,
            React.createElement(CheckCircle_1.default, { style: { color: color.success, height: 18, width: 18 } })));
    }
    else if (props.notebookValidation === false) {
        validationTpl = (React.createElement(React.Fragment, null,
            React.createElement(Error_1.default, { style: { color: color.errorText, height: 18, width: 18 } })));
    }
    else {
        validationTpl = React.createElement(core_1.LinearProgress, { color: "primary" });
    }
    let compileTpl;
    if (props.compiledPath && props.compiledPath !== 'error') {
        compileTpl = (React.createElement(React.Fragment, null,
            React.createElement("a", { onClick: _ => props.docManager.openOrReveal(props.compiledPath) },
                "Done",
                React.createElement(CheckCircle_1.default, { style: { color: color.success, height: 18, width: 18 } }))));
    }
    else if (props.compiledPath === 'error') {
        compileTpl = (React.createElement(React.Fragment, null,
            React.createElement(Error_1.default, { style: { color: color.errorText, height: 18, width: 18 } })));
    }
    else {
        compileTpl = React.createElement(core_1.LinearProgress, { color: "primary" });
    }
    let uploadTpl;
    if (props.pipeline) {
        uploadTpl = (React.createElement(React.Fragment, null,
            React.createElement("a", { href: getUploadLink(props.pipeline), target: "_blank", rel: "noopener noreferrer" },
                "Done",
                React.createElement(Launch_1.default, { style: { height: 18, width: 18 } }))));
    }
    else if (props.pipeline === false) {
        uploadTpl = (React.createElement(React.Fragment, null,
            React.createElement(Error_1.default, { style: { color: color.errorText, height: 18, width: 18 } })));
    }
    else {
        uploadTpl = React.createElement(core_1.LinearProgress, { color: "primary" });
    }
    let runTpl;
    if (props.runPipeline) {
        runTpl = (React.createElement(React.Fragment, null,
            React.createElement("a", { href: getRunLink(props.runPipeline), target: "_blank", rel: "noopener noreferrer" }, getRunComponent(props.runPipeline))));
    }
    else if (props.runPipeline == false) {
        runTpl = (React.createElement(React.Fragment, null,
            React.createElement(Error_1.default, { style: { color: color.errorText, height: 18, width: 18 } })));
    }
    else {
        runTpl = React.createElement(core_1.LinearProgress, { color: "primary" });
    }
    let katibTpl;
    if (!props.katib) {
        katibTpl = React.createElement(core_1.LinearProgress, { color: "primary" });
    }
    else if (props.katib.status == 'error') {
        katibTpl = (React.createElement(React.Fragment, null,
            React.createElement(Error_1.default, { style: { color: color.errorText, height: 18, width: 18 } })));
    }
    else {
        katibTpl = (React.createElement(React.Fragment, null,
            React.createElement("a", { href: getKatibLink(props.katib), target: "_blank", rel: "noopener noreferrer" }, getKatibComponent(props.katib))));
    }
    let katibKfpExpTpl;
    if (!props.katibKFPExperiment) {
        katibKfpExpTpl = React.createElement(core_1.LinearProgress, { color: "primary" });
    }
    else if (props.katibKFPExperiment.id !== 'error') {
        katibKfpExpTpl = (React.createElement(React.Fragment, null,
            React.createElement("a", { href: getKatibKfpExperimentLink(props.katibKFPExperiment.id), target: "_blank", rel: "noopener noreferrer" },
                "Done",
                React.createElement(Launch_1.default, { style: { fontSize: '1rem' } }))));
    }
    else {
        katibKfpExpTpl = (React.createElement(React.Fragment, null,
            React.createElement(Error_1.default, { style: { color: color.errorText, height: 18, width: 18 } })));
    }
    let katibRunsTpl = undefined;
    if (!props.katib || props.katib.trials === 0) {
        katibRunsTpl = (React.createElement(React.Fragment, null,
            React.createElement("div", { className: "deploy-progress-label" }, "Gathering suggestions..."),
            React.createElement("div", { className: "deploy-progress-value" },
                React.createElement(core_1.LinearProgress, { color: "primary" }))));
    }
    else if (props.katib.status !== 'error') {
        // we have some katib trials
        katibRunsTpl = (React.createElement(React.Fragment, null,
            React.createElement("div", { className: "deploy-progress-label labels-indented" },
                React.createElement("p", null,
                    "Running: ",
                    props.katib.trialsRunning),
                React.createElement("p", null,
                    "Succeeded: ",
                    props.katib.trialsSucceeded),
                React.createElement("p", null,
                    "Failed: ",
                    props.katib.trialsFailed))));
    }
    return (React.createElement("div", { className: "deploy-progress" },
        React.createElement("div", { style: {
                justifyContent: 'flex-end',
                textAlign: 'right',
                paddingRight: '4px',
                height: '1rem',
            } },
            React.createElement(Close_1.default, { style: { fontSize: '1rem', cursor: 'pointer' }, onClick: _ => props.onRemove() })),
        props.showValidationProgress ? (React.createElement("div", { className: "deploy-progress-row" },
            React.createElement("div", { className: "deploy-progress-label" }, "Validating notebook..."),
            React.createElement("div", { className: "deploy-progress-value" },
                validationTpl,
                getWarningBadge('Validation Warnings', props.validationWarnings)))) : null,
        props.showSnapshotProgress ? (React.createElement("div", { className: "deploy-progress-row" },
            React.createElement("div", { className: "deploy-progress-label" }, "Taking snapshot..."),
            React.createElement("div", { className: "deploy-progress-value" },
                getSnapshotTpl(),
                ' ',
                getWarningBadge('Snapshot Warnings', props.snapshotWarnings)))) : null,
        props.showCompileProgress ? (React.createElement("div", { className: "deploy-progress-row" },
            React.createElement("div", { className: "deploy-progress-label" }, "Compiling notebook..."),
            React.createElement("div", { className: "deploy-progress-value" },
                compileTpl,
                getWarningBadge('Compile Warnings', props.compileWarnings)))) : null,
        props.showUploadProgress ? (React.createElement("div", { className: "deploy-progress-row" },
            React.createElement("div", { className: "deploy-progress-label" }, "Uploading pipeline..."),
            React.createElement("div", { className: "deploy-progress-value" },
                uploadTpl,
                getWarningBadge('Upload Warnings', props.uploadWarnings)))) : null,
        props.showRunProgress ? (React.createElement("div", { className: "deploy-progress-row" },
            React.createElement("div", { className: "deploy-progress-label" }, "Running pipeline..."),
            React.createElement("div", { className: "deploy-progress-value" },
                runTpl,
                getWarningBadge('Run Warnings', props.runWarnings)))) : null,
        props.showKatibKFPExperiment ? (React.createElement("div", { className: "deploy-progress-row" },
            React.createElement("div", { className: "deploy-progress-label" }, "Creating KFP experiment..."),
            React.createElement("div", { className: "deploy-progress-value" }, katibKfpExpTpl))) : null,
        props.showKatibProgress ? (React.createElement("div", null,
            React.createElement("div", { className: "deploy-progress-row", style: { borderBottom: 'transparent', paddingBottom: 0 } },
                React.createElement("div", { className: "deploy-progress-label" }, "Running Katib experiment..."),
                React.createElement("div", { className: "deploy-progress-value" },
                    katibTpl,
                    getInfoBadge('Katib current best result', getKatibBestResultInfo(props.katib)))),
            React.createElement("div", { className: "deploy-progress-row" }, katibRunsTpl))) : null));
};
